/******************************************************************************
 *
 * Copyright:
 *    (C) 2000 - 2009 Embedded Artists AB
 *
 * File:
 *    main.c
 *
 * Description:
 *    Sample application that demonstrates how to control a 7-segment directly
 *    via GPIO:s (General Purpose I/O pins). Count down when pressing P1.20-key
 *    and count up when pressing P1.23-key.  
 *
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "general.h"
#include <lpc2xxx.h>
#include <printf_P.h>
#include <ea_init.h>
#include "startup/config.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/
#define SEVEN_SEG_A   0x00000100
#define SEVEN_SEG_B   0x00000200
#define SEVEN_SEG_C   0x00000400
#define SEVEN_SEG_D   0x00000800
#define SEVEN_SEG_E   0x00001000
#define SEVEN_SEG_F   0x00002000
#define SEVEN_SEG_G   0x00004000
#define SEVEN_SEG_DOT 0x00008000

#define KEY_P1_20     0x00100000
#define KEY_P1_23     0x00800000


/*****************************************************************************
 * Public function prototypes
 ****************************************************************************/
int main(void);


/*****************************************************************************
 * Local function prototypes
 ****************************************************************************/
static void delayMs(tU16 delayInMs);
static void set7seg(tU8 value);


/*****************************************************************************
 * Implementation of local functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    Delay execution by a specified number of milliseconds by using
 *    timer #1. A polled implementation.
 *
 * Params:
 *    [in] delayInMs - the number of milliseconds to delay.
 *
 ****************************************************************************/
static void
delayMs(tU16 delayInMs)
{
  /*
   * setup timer #1 for delay
   */
  TIMER1_TCR = 0x02;          //stop and reset timer
  TIMER1_PR  = 0x00;          //set prescaler to zero
  TIMER1_MR0 = delayInMs * ((FOSC * PLL_MUL)/ (1000 * PBSD));
  TIMER1_IR  = 0xff;          //reset all interrrupt flags
  TIMER1_MCR = 0x04;          //stop timer on match
  TIMER1_TCR = 0x01;          //start timer
  
  //wait until delay time has elapsed
  while (TIMER1_TCR & 0x01)
    ;
}


/*****************************************************************************
 *
 * Description:
 *
 ****************************************************************************/
static void
set7seg(tU8 value)
{
  IOCLR0 = (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D | SEVEN_SEG_E | SEVEN_SEG_F | SEVEN_SEG_G);
  switch(value)
  {
    case  0: IOSET0 = (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D | SEVEN_SEG_E | SEVEN_SEG_F              ); break;
    case  1: IOSET0 = (              SEVEN_SEG_B | SEVEN_SEG_C                                                        ); break;
    case  2: IOSET0 = (SEVEN_SEG_A | SEVEN_SEG_B               | SEVEN_SEG_D | SEVEN_SEG_E               | SEVEN_SEG_G); break;
    case  3: IOSET0 = (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D                             | SEVEN_SEG_G); break;
    case  4: IOSET0 = (              SEVEN_SEG_B | SEVEN_SEG_C                             | SEVEN_SEG_F | SEVEN_SEG_G); break;
    case  5: IOSET0 = (SEVEN_SEG_A               | SEVEN_SEG_C | SEVEN_SEG_D               | SEVEN_SEG_F | SEVEN_SEG_G); break;
    case  6: IOSET0 = (SEVEN_SEG_A               | SEVEN_SEG_C | SEVEN_SEG_D | SEVEN_SEG_E | SEVEN_SEG_F | SEVEN_SEG_G); break;
    case  7: IOSET0 = (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C                                                        ); break;
    case  8: IOSET0 = (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D | SEVEN_SEG_E | SEVEN_SEG_F | SEVEN_SEG_G); break;
    case  9: IOSET0 = (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D               | SEVEN_SEG_F | SEVEN_SEG_G); break;
    default: break;
  }
}


/*****************************************************************************
 * Implementation of public functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    The main-function. 
 *
 * Returns:
 *    Always 0, since return value is not used.
 *
 ****************************************************************************/
int
main(void)
{
	tS32 keyCount = 0;
	tU32 lastKeyState;

  //initialize printf()-functionality
  eaInit();

  //print welcome message
  printf("\n*********************************************************");
  printf("\n*");
  printf("\n* (C) 2000 - 2009 Embedded Artists AB");
  printf("\n*");
  printf("\n* Excersise #1d - direct drive of 7-segment display ");
  printf("\n* via GPIO:s. Count down when pressing P1.20-key");
  printf("\n* and count up when pressing P1.23-key.");
  printf("\n*");
  printf("\n*********************************************************");

  //Set controlling pins to outputs and set them low (= LEDs are off)
  IODIR0 |= (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D |
             SEVEN_SEG_E | SEVEN_SEG_F | SEVEN_SEG_G | SEVEN_SEG_DOT);
  IOCLR0  = (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D |
             SEVEN_SEG_E | SEVEN_SEG_F | SEVEN_SEG_G | SEVEN_SEG_DOT);

  //Set P1.20 and P1.23 keys as inputs
  IODIR1 &= ~(KEY_P1_20 | KEY_P1_23);

  //Set start value on display
  set7seg(keyCount);
  
  //init state variable
  lastKeyState = (IOPIN1 & (KEY_P1_20 | KEY_P1_23));
  
  while(1)
  {
    //Check if key P1.20 is pressed and last time it was not
	  if (((IOPIN1 & KEY_P1_20) == 0) && ((lastKeyState & KEY_P1_20) != 0))
 	  {
 	    //update counter and check boundary value
      keyCount--;
      if (keyCount < 0)
        keyCount = 9;
      
      //Update 7-segment display
      set7seg(keyCount);
 	  }

    //Check if key P1.23 is pressed and last time it was not
	  if (((IOPIN1 & KEY_P1_23) == 0) && ((lastKeyState & KEY_P1_23) != 0))
 	  {
 	    //update counter and check boundary value
      keyCount++;
      if (keyCount >= 10)
        keyCount = 0;
      
      //Update 7-segment display
      set7seg(keyCount);
 	  }

    //init state variable
    lastKeyState = (IOPIN1 & (KEY_P1_20 | KEY_P1_23));

    //wait 10 ms
    delayMs(10);    
  }

  return 0;
}
