/******************************************************************************
 *
 * Copyright:
 *    (C) 2000 - 2009 Embedded Artists AB
 *
 * File:
 *    main.c
 *
 * Description:
 *    Sample application that demonstrates how to control a 7-segment directly
 *    via GPIO:s (General Purpose I/O pins). Count up and down when pressing
 *    the touch keys.  
 *
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "general.h"
#include <lpc2xxx.h>
#include <printf_P.h>
#include <ea_init.h>
#include "startup/config.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/
#define SEVEN_SEG_A   0x00000100
#define SEVEN_SEG_B   0x00000200
#define SEVEN_SEG_C   0x00000400
#define SEVEN_SEG_D   0x00000800
#define SEVEN_SEG_E   0x00001000
#define SEVEN_SEG_F   0x00002000
#define SEVEN_SEG_G   0x00004000
#define SEVEN_SEG_DOT 0x00008000

#define TOUCH_P1_24   (1<<24)
#define TOUCH_P1_25   (1<<25)

#define DOWN_KEY      1
#define UP_KEY        2


/*****************************************************************************
 * Public function prototypes
 ****************************************************************************/
int main(void);


/*****************************************************************************
 * Local function prototypes
 ****************************************************************************/
static void delayMs(tU16 delayInMs);
static void set7seg(tU8 value);


/*****************************************************************************
 * Implementation of local functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    Delay execution by a specified number of milliseconds by using
 *    timer #1. A polled implementation.
 *
 * Params:
 *    [in] delayInMs - the number of milliseconds to delay.
 *
 ****************************************************************************/
static void
delayMs(tU16 delayInMs)
{
  /*
   * setup timer #1 for delay
   */
  TIMER1_TCR = 0x02;          //stop and reset timer
  TIMER1_PR  = 0x00;          //set prescaler to zero
  TIMER1_MR0 = delayInMs * ((FOSC * PLL_MUL)/ (1000 * PBSD));
  TIMER1_IR  = 0xff;          //reset all interrrupt flags
  TIMER1_MCR = 0x04;          //stop timer on match
  TIMER1_TCR = 0x01;          //start timer
  
  //wait until delay time has elapsed
  while (TIMER1_TCR & 0x01)
    ;
}


/*****************************************************************************
 *
 * Description:
 *
 ****************************************************************************/
static void
set7seg(tU8 value)
{
  IOCLR0 = (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D | SEVEN_SEG_E | SEVEN_SEG_F | SEVEN_SEG_G);
  switch(value)
  {
    case  0: IOSET0 = (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D | SEVEN_SEG_E | SEVEN_SEG_F              ); break;
    case  1: IOSET0 = (              SEVEN_SEG_B | SEVEN_SEG_C                                                        ); break;
    case  2: IOSET0 = (SEVEN_SEG_A | SEVEN_SEG_B               | SEVEN_SEG_D | SEVEN_SEG_E               | SEVEN_SEG_G); break;
    case  3: IOSET0 = (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D                             | SEVEN_SEG_G); break;
    case  4: IOSET0 = (              SEVEN_SEG_B | SEVEN_SEG_C                             | SEVEN_SEG_F | SEVEN_SEG_G); break;
    case  5: IOSET0 = (SEVEN_SEG_A               | SEVEN_SEG_C | SEVEN_SEG_D               | SEVEN_SEG_F | SEVEN_SEG_G); break;
    case  6: IOSET0 = (SEVEN_SEG_A               | SEVEN_SEG_C | SEVEN_SEG_D | SEVEN_SEG_E | SEVEN_SEG_F | SEVEN_SEG_G); break;
    case  7: IOSET0 = (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C                                                        ); break;
    case  8: IOSET0 = (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D | SEVEN_SEG_E | SEVEN_SEG_F | SEVEN_SEG_G); break;
    case  9: IOSET0 = (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D               | SEVEN_SEG_F | SEVEN_SEG_G); break;
    default: break;
  }
}


/*****************************************************************************
 *
 * Description:
 *   Code below is for reading touch keys 
 *
 ****************************************************************************/
typedef enum _ButtonError_t
{
  BUTT_OK = 0, BUTT_TO_ERROR, BUTT_SHORT_ERROR
} ButtonError_t;

#define _BUT_MAX_SCAN_PER   200
#define _BUT_MIN_SCAN_PER   10

#define _PDIR_OFFSET        (((unsigned int)&IODIR0 - (unsigned int)&IOPIN0)/sizeof(unsigned int))
#define _PSET_OFFSET        (((unsigned int)&IOSET0 - (unsigned int)&IOPIN0)/sizeof(unsigned int))
#define _PCLR_OFFSET        (((unsigned int)&IOCLR0 - (unsigned int)&IOPIN0)/sizeof(unsigned int))
#define _PIN_OFFSET         (((unsigned int)&IOPIN0 - (unsigned int)&IOPIN0)/sizeof(unsigned int))

#define _PORT1_BUT1_BIT     24
#define _PORT1_BUT2_BIT     25

typedef struct _ButtonCtrl_t
{
  volatile unsigned int * pButBaseReg;
           unsigned int ButBit;
} ButtonCtrl_t, *pButtonCtrl_t;

typedef struct _ButtonsPairCtrl_t
{
  ButtonCtrl_t Ba;
  ButtonCtrl_t Bb;
} ButtonsPairCtrl_t, *pButtonsPairCtrl_t;

const ButtonsPairCtrl_t ButtonsCtrl[1] =
{
  {
    .Ba=
    {
      .pButBaseReg = &IOPIN1,   // CAP_BUTT_1 = P1.24
      .ButBit = _PORT1_BUT1_BIT,
    },
    .Bb=
    {
      .pButBaseReg = &IOPIN1,   // CAP_BUTT_2 = P1.25
      .ButBit = _PORT1_BUT2_BIT,
    }
  }
};

tU8
readTouch(tU8 id, tU32 *pCount)
{
  volatile unsigned int To;
  unsigned int Count, Hold, MasterMask, SlaveMask;
  volatile unsigned int *pMasterReg, *pSlaveReg;

  To = _BUT_MAX_SCAN_PER;

  if(id & 1)
  {
    pMasterReg = ButtonsCtrl[id>>1].Ba.pButBaseReg;
    MasterMask = 1UL << ButtonsCtrl[id>>1].Ba.ButBit;
    pSlaveReg  = ButtonsCtrl[id>>1].Bb.pButBaseReg;
    SlaveMask  = 1UL << ButtonsCtrl[id>>1].Bb.ButBit;
  }
  else
  {
    pMasterReg = ButtonsCtrl[id>>1].Bb.pButBaseReg;
    MasterMask = 1UL << ButtonsCtrl[id>>1].Bb.ButBit;
    pSlaveReg  = ButtonsCtrl[id>>1].Ba.pButBaseReg;
    SlaveMask  = 1UL << ButtonsCtrl[id>>1].Ba.ButBit;
  }

  // 5. Ba o0
  *(pSlaveReg + _PCLR_OFFSET)  = SlaveMask;
  *(pSlaveReg + _PDIR_OFFSET) |= SlaveMask;

  // 6. Set Ba i
  *(pSlaveReg + _PDIR_OFFSET) &= ~SlaveMask;

  // 7. Set Bb o1
  //disable interrupts (if needed)
  T1TCR = 1; // enable Timer
  Count = T1TC;
  *(pMasterReg + _PSET_OFFSET) = MasterMask;

  // 8. wait and counting until Ba state get 1
  while(!(*(pSlaveReg + _PIN_OFFSET) & SlaveMask))
  {
    if(!To)
    {
      break;
    }
    --To;
  }
//  Hold += T1TC - Count;
  Hold = T1TC - Count;
  //enable interrupts again (if needed)
  T1TCR = 0;  // disable Timer

  // 9. Set Ba o1
  *(pSlaveReg + _PSET_OFFSET)  = SlaveMask;
  *(pSlaveReg + _PDIR_OFFSET) |= SlaveMask;

  if(!To)
  {
    return(BUTT_TO_ERROR);
  }

  if(To == _BUT_MAX_SCAN_PER)
  {
    return(BUTT_SHORT_ERROR);
  }

  *pCount = Hold;
  return(BUTT_OK);
}


/*****************************************************************************
 * Implementation of public functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    The main-function. 
 *
 * Returns:
 *    Always 0, since return value is not used.
 *
 ****************************************************************************/
int
main(void)
{
	tS32 keyCount = 0;
	tU32 lastKeyState;

  //initialize printf()-functionality
  eaInit();

  //print welcome message
  printf("\n*********************************************************");
  printf("\n*");
  printf("\n* (C) 2000 - 2009 Embedded Artists AB");
  printf("\n*");
  printf("\n* Excersise #1e - direct drive of 7-segment display ");
  printf("\n* via GPIO:s. Count up and down when pressing the");
  printf("\n* touch keys. "); 
  printf("\n*");
  printf("\n*********************************************************");

  //Set controlling pins to outputs and set them low (= LEDs are off)
  IODIR0 |= (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D |
             SEVEN_SEG_E | SEVEN_SEG_F | SEVEN_SEG_G | SEVEN_SEG_DOT);
  IOCLR0  = (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D |
             SEVEN_SEG_E | SEVEN_SEG_F | SEVEN_SEG_G | SEVEN_SEG_DOT);

  //Set P1.20 and P1.23 keys as inputs
  IOSET1  =  (TOUCH_P1_24 | TOUCH_P1_25);
  IODIR1 &= ~(TOUCH_P1_24 | TOUCH_P1_25);

  //init timer for reading touch keys
  T1TCR = 0;            // counter disable
  T1PR  = 0;            // set prescaler /1
  T1MCR = 0;            // disable match act
  T1EMR = 0;            // disable external match act

  //Set start value on display
  set7seg(keyCount);
  
  //init state variable
  lastKeyState = 0;

  while(1)
  {
    tU32 dummy;

    if (BUTT_OK == readTouch(0, &dummy))
    {
      if ((lastKeyState & DOWN_KEY) == 0)
      {
 	      //update counter and check boundary value
        keyCount--;
        if (keyCount < 0)
          keyCount = 9;
      
        //Update 7-segment display
        set7seg(keyCount);
      }
      lastKeyState |= DOWN_KEY;
    }
    else
      lastKeyState &= ~DOWN_KEY;
    
    if (BUTT_OK == readTouch(1, &dummy))
    {
      if ((lastKeyState & UP_KEY) == 0)
      {
 	      //update counter and check boundary value
        keyCount++;
        if (keyCount >= 10)
          keyCount = 0;
      
        //Update 7-segment display
        set7seg(keyCount);
      }
      lastKeyState |= UP_KEY;
    }
    else
      lastKeyState &= ~UP_KEY;

    //wait 10 ms
    delayMs(10);    
  }

  return 0;
}
