/******************************************************************************
 *
 * Copyright:
 *    (C) 2000 - 2009 Embedded Artists AB
 *
 * File:
 *    main.c
 *
 * Description:
 *    Sample application that demonstrates how to control a 7-segment directly
 *    via GPIO:s (General Purpose I/O pins). Analog input controls display value.
 *
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "general.h"
#include <lpc2xxx.h>
#include <printf_P.h>
#include <ea_init.h>
#include "startup/config.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/
#define SEVEN_SEG_A   0x00000100
#define SEVEN_SEG_B   0x00000200
#define SEVEN_SEG_C   0x00000400
#define SEVEN_SEG_D   0x00000800
#define SEVEN_SEG_E   0x00001000
#define SEVEN_SEG_F   0x00002000
#define SEVEN_SEG_G   0x00004000
#define SEVEN_SEG_DOT 0x00008000

#define AIN1 1
#define AIN2 2


/*****************************************************************************
 * Public function prototypes
 ****************************************************************************/
int main(void);


/*****************************************************************************
 * Local function prototypes
 ****************************************************************************/
static void delayMs(tU16 delayInMs);
static void set7seg(tU8 value);


/*****************************************************************************
 * Implementation of local functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    Delay execution by a specified number of milliseconds by using
 *    timer #1. A polled implementation.
 *
 * Params:
 *    [in] delayInMs - the number of milliseconds to delay.
 *
 ****************************************************************************/
static void
delayMs(tU16 delayInMs)
{
  /*
   * setup timer #1 for delay
   */
  TIMER1_TCR = 0x02;          //stop and reset timer
  TIMER1_PR  = 0x00;          //set prescaler to zero
  TIMER1_MR0 = delayInMs * ((FOSC * PLL_MUL)/ (1000 * PBSD));
  TIMER1_IR  = 0xff;          //reset all interrrupt flags
  TIMER1_MCR = 0x04;          //stop timer on match
  TIMER1_TCR = 0x01;          //start timer
  
  //wait until delay time has elapsed
  while (TIMER1_TCR & 0x01)
    ;
}


/*****************************************************************************
 *
 * Description:
 *
 ****************************************************************************/
static void
set7seg(tU8 value)
{
  IOCLR0 = (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D | SEVEN_SEG_E | SEVEN_SEG_F | SEVEN_SEG_G);
  switch(value)
  {
    case  0: IOSET0 = (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D | SEVEN_SEG_E | SEVEN_SEG_F              ); break;
    case  1: IOSET0 = (              SEVEN_SEG_B | SEVEN_SEG_C                                                        ); break;
    case  2: IOSET0 = (SEVEN_SEG_A | SEVEN_SEG_B               | SEVEN_SEG_D | SEVEN_SEG_E               | SEVEN_SEG_G); break;
    case  3: IOSET0 = (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D                             | SEVEN_SEG_G); break;
    case  4: IOSET0 = (              SEVEN_SEG_B | SEVEN_SEG_C                             | SEVEN_SEG_F | SEVEN_SEG_G); break;
    case  5: IOSET0 = (SEVEN_SEG_A               | SEVEN_SEG_C | SEVEN_SEG_D               | SEVEN_SEG_F | SEVEN_SEG_G); break;
    case  6: IOSET0 = (SEVEN_SEG_A               | SEVEN_SEG_C | SEVEN_SEG_D | SEVEN_SEG_E | SEVEN_SEG_F | SEVEN_SEG_G); break;
    case  7: IOSET0 = (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C                                                        ); break;
    case  8: IOSET0 = (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D | SEVEN_SEG_E | SEVEN_SEG_F | SEVEN_SEG_G); break;
    case  9: IOSET0 = (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D               | SEVEN_SEG_F | SEVEN_SEG_G); break;
    default: break;
  }
}

/*****************************************************************************
 *
 * Description:
 *    Start a conversion of one selected analogue input and return
 *    10-bit result.
 *
 * Params:
 *    [in] channel - analogue input channel to convert.
 *
 * Return:
 *    10-bit conversion result
 *
 ****************************************************************************/
tU16
getAnalogueInput(tU8 channel)
{
  tU16 returnResult;

	//start conversion now (for selected channel)
	ADCR = (ADCR & 0xFFFFFF00) | (1 << channel) | (1 << 24);
	
	//wait til done
	while((ADDR & 0x80000000) == 0)
	  ;

	//get result and adjust to 10-bit integer
	returnResult = (ADDR>>6) & 0x3FF;

  return returnResult;
}

/*****************************************************************************
 *
 * Description:
 *
 ****************************************************************************/
void
initAdc(void)
{
	volatile tU32 integerResult;

	//
  //Initialize ADC: AIN1 = P0.28
  //
  PINSEL1 &= ~0x03000000;
  PINSEL1 |=  0x01000000;

	//
  //Initialize ADC: AIN2 = P0.29
  //
  PINSEL1 &= ~0x0C000000;
  PINSEL1 |=  0x04000000;

  //initialize ADC
  ADCR = (1 << 0)                                     |  //SEL = 1, dummy channel #0
         ((FOSC * PLL_MUL / PBSD) / 4500000 - 1) << 8 |  //set clock division factor, so ADC clock is 4.5MHz
         (0 << 16)                                    |  //BURST = 0, conversions are SW controlled
         (0 << 17)                                    |  //CLKS  = 0, 11 clocks = 10-bit result
         (1 << 21)                                    |  //PDN   = 1, ADC is active
         (1 << 24)                                    |  //START = 1, start a conversion now
         (0 << 27);							                         //EDGE  = 0, not relevant when start=1

  //short delay and dummy read
  delayMs(10);
  integerResult = ADDR;
}


/*****************************************************************************
 * Implementation of public functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    The main-function. 
 *
 * Returns:
 *    Always 0, since return value is not used.
 *
 ****************************************************************************/
int
main(void)
{
  //initialize printf()-functionality
  eaInit();

  //print welcome message
  printf("\n*********************************************************");
  printf("\n*");
  printf("\n* (C) 2000 - 2009 Embedded Artists AB");
  printf("\n*");
  printf("\n* Excersise #1f - direct drive of 7-segment display ");
  printf("\n* via GPIO:s. Analog input controls display value.");
  printf("\n*");
  printf("\n*********************************************************");

  //Set controlling pins to outputs and set them low (= LEDs are off)
  IODIR0 |= (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D |
             SEVEN_SEG_E | SEVEN_SEG_F | SEVEN_SEG_G | SEVEN_SEG_DOT);
  IOCLR0  = (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D |
             SEVEN_SEG_E | SEVEN_SEG_F | SEVEN_SEG_G | SEVEN_SEG_DOT);

  //Initialize ADC
  initAdc();
  
  while(1)
  {
    tU16 adValue;
    static tU8 message[] = "Analog in: xxxx";

    //read analog input (adValue varies from 0-1023)
    adValue = getAnalogueInput(AIN1);

    //Create string with analog value
    message[14] = ((adValue / 1)    % 10) + '0';
    message[13] = ((adValue / 10)   % 10) + '0';
    message[12] = ((adValue / 100)  % 10) + '0';
    message[11] = ((adValue / 1000) % 10) + '0';
    if (message[11] == '0')
    {
      message[11] = ' ';
      if (message[12] == '0')
      {
        message[12] = ' ';
        if (message[13] == '0')
          message[13] = ' ';
      }
    }
    printf("\n%s", message);
      
    //Adjust value for 7-segment display and output    
    adValue = adValue / 102;
    if (adValue >= 10)
      adValue = 9;
    
    set7seg(adValue);

    //wait 50 ms
    delayMs(50);    

  }

  return 0;
}
