/******************************************************************************
 *
 * Copyright:
 *    (C) 2000 - 2009 Embedded Artists AB
 *
 * File:
 *    main.c
 *
 * Description:
 *    Sample application that demonstrates how to control a 7-segment via
 *    a decoder chip. Count with timer interrupt.
 *
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "general.h"
#include <lpc2xxx.h>
#include <printf_P.h>
#include <ea_init.h>
#include "startup/config.h"
#include "irq_code/irq.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/
#define DECODER_VALUE 0x00000F00
#define DECODER_LT    0x00001000
#define DECODER_BI    0x00002000
#define DECODER_LE    0x00004000

#define SEVEN_SEG_DOT 0x00008000

#define KEY_P1_20     0x00100000


/******************************************************************************
 * Local variables
 *****************************************************************************/
volatile tU32 systemTimeMs;
void (*pTimerCallback)(void);

/*****************************************************************************
 * Public function prototypes
 ****************************************************************************/
int main(void);
tU32 getSystemTime(void);

/*****************************************************************************
 * Local function prototypes
 ****************************************************************************/
static void delayMs(tU16 delayInMs);
static void set7seg(tU8 value);
static void initVIC(void (*pDefaultISR)());
static void initTimer0(void (*pISR)(), void (*pCallback)());

/*****************************************************************************
 * Implementation of local functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    This function initialize the Vectored Interrupt Controller (VIC) to
 *    a known state. All interrupts are vectored to a default interrupt
 *    handler.
 *
 * Params:
 *    [in] pDefaultISR - function pointer to the default interrupt ISR.
 *
 ****************************************************************************/
static void
initVIC(void (*pDefaultISR)())
{
  VICIntSelect   = 0x00000000;         //all IRQs are assigned to IRQ (not FIQ)
  VICIntEnClr    = 0xffffffff;         //disable all interrupts
  VICSoftIntClr  = 0xffffffff;         //clear all pending software interrupts
  VICProtection  = 0x00000000;         //VIC protection disabled, registers can
                                       // be accessed in User and Privileged mode
  VICVectAddr    = 0x00000000;         //clear any pending/active interrupts
  VICDefVectAddr = (tU32)pDefaultISR;  //set default ISR address for for non-vectored IRQs
  
  //reset/disable all vectored IRQ slots
  VICVectCntl0   = 0x00000000;
  VICVectCntl1   = 0x00000000;
  VICVectCntl2   = 0x00000000;
  VICVectCntl3   = 0x00000000;
  VICVectCntl4   = 0x00000000;
  VICVectCntl5   = 0x00000000;
  VICVectCntl6   = 0x00000000;
  VICVectCntl7   = 0x00000000;
  VICVectCntl8   = 0x00000000;
  VICVectCntl9   = 0x00000000;
  VICVectCntl10  = 0x00000000;
  VICVectCntl11  = 0x00000000;
  VICVectCntl12  = 0x00000000;
  VICVectCntl13  = 0x00000000;
  VICVectCntl14  = 0x00000000;
  VICVectCntl15  = 0x00000000;
}

/*****************************************************************************
 *
 * Description:
 *    Initialize Timer #0 (and VIC) for generating system tick interrupts.
 *
 * Params:
 *    [in] pISR      - function pointer to timer ISR.
 *    [in] pCallback - function pointer to timer callback function.
 *                     NULL if no callback function shall be registered.
 *
 ****************************************************************************/
static void
initTimer0(void (*pISR)(), void (*pCallback)())
{
  //store timer callback function pointer
  pTimerCallback = pCallback;
  
  //initialize VIC for Timer0 interrupts
  VICIntSelect &= ~0x10;       //Timer0 interrupt is assigned to IRQ (not FIQ)
  VICVectAddr4  = (tU32)pISR;  //register ISR address
  VICVectCntl4  = 0x24;        //enable vector interrupt for timer0
  VICIntEnable  = 0x10;        //enable timer0 interrupt
  
  //initialize and start Timer0
  T0TCR = 0x00000002;                           //disable and reset Timer0
  T0PC  = 0x00000000;                           //no prescale of clock
  T0MR0 = (FOSC * PLL_MUL) /                    //calculate no of timer ticks
          (SYSTEM_TIME_TICK * PBSD);            //for a given system tick rate
  T0IR  = 0x000000ff;                           //reset all flags before enable IRQs
  T0MCR = 0x00000003;                           //reset counter and generate IRQ on MR0 match
  T0TCR = 0x00000001;                           //start Timer0
  
  //reset system time
  systemTimeMs = 0;
}


/*****************************************************************************
 *
 * Description:
 *    A callback function that is called every time a timer interrupt occurs.
 *    Observe that the function executes in interrupt context.
 *
 ****************************************************************************/
static void
myCallback(void)
{
  //Remember that this function will be executed in interrupt
  //context so keep it short... printf():s are generally not a good idea...

	static tU16 counter = 0;
	
  //update counter - count to 12 to get 500ms delay (tick rate is 25Hz)
  counter++;
  if ((counter % 12) == 0)
  {
    //check if counter has reached max (count 12 for every increment on 7-segment display)
    if (counter >= 12*10)
      counter = 0;
       
    set7seg(counter / 12);
  }
}

/*****************************************************************************
 *
 * Description:
 *    Delay execution by a specified number of milliseconds by using
 *    timer #1. A polled implementation.
 *
 * Params:
 *    [in] delayInMs - the number of milliseconds to delay.
 *
 ****************************************************************************/
static void
delayMs(tU16 delayInMs)
{
  /*
   * setup timer #1 for delay
   */
  TIMER1_TCR = 0x02;          //stop and reset timer
  TIMER1_PR  = 0x00;          //set prescaler to zero
  TIMER1_MR0 = delayInMs * ((FOSC * PLL_MUL)/ (1000 * PBSD));
  TIMER1_IR  = 0xff;          //reset all interrrupt flags
  TIMER1_MCR = 0x04;          //stop timer on match
  TIMER1_TCR = 0x01;          //start timer
  
  //wait until delay time has elapsed
  while (TIMER1_TCR & 0x01)
    ;
}


/*****************************************************************************
 *
 * Description:
 *
 ****************************************************************************/
static void
set7seg(tU8 value)
{
  //output value
  IOCLR0 = DECODER_VALUE;
  IOSET0 = ((value & 0x0f) << 8);

  //set latch low
  IOCLR0 = DECODER_LE;

  //set latch high to store value
  IOSET0 = DECODER_LE;

}


/*****************************************************************************
 * Implementation of public functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    Returns the current number of ms from system start.
 *
 * Returns:
 *    tU32 - number of milliseconds since system start.
 *
 ****************************************************************************/
tU32
getSystemTime(void)
{
  return systemTimeMs;
}

/*****************************************************************************
 *
 * Description:
 *    The main-function. 
 *
 * Returns:
 *    Always 0, since return value is not used.
 *
 ****************************************************************************/
int
main(void)
{
  //initialize printf()-functionality
  eaInit();

  //print welcome message
  printf("\n*********************************************************");
  printf("\n*");
  printf("\n* (C) 2000 - 2009 Embedded Artists AB");
  printf("\n*");
  printf("\n* Excersise #2a - drive of 7-segment display via decoder.");
  printf("\n* Count with timer interrupt.");
  printf("\n*");
  printf("\n*********************************************************");

  //Set controlling pins to outputs and set them to default values
  IODIR0 |= (DECODER_VALUE | DECODER_LT | DECODER_BI | DECODER_LE | SEVEN_SEG_DOT);
  IOSET0  =  DECODER_LT;  //Lamp test active low
  IOSET0  =  DECODER_BI;
  IOSET0  =  DECODER_LE;
  IOCLR0  =  SEVEN_SEG_DOT;

  //Set P1.20 key as input
  IODIR1 &= ~KEY_P1_20;

  //Set start value on display
  set7seg(0);

  //initialize the VIC and register the default ISR handler
  initVIC(dummyISR);
  
  //initialize Timer #0 and start system time counting
  initTimer0(timer0ISR, myCallback);

  while(1)
  {
    tU32 currentTime;
    
    //pause for 200 ms by taking a time stamp
    currentTime = getSystemTime();
    while ((getSystemTime() - currentTime) < 200)
      ;
      
    //activate lamp test when P1.20 key is pressed
    printf("\nCheck P1.20 key");
	  if ((IOPIN1 & KEY_P1_20) == 0)
 	  {
      IOCLR0 = DECODER_LT; //Lamp test active low
 	  }
 	  else
 	  {
      IOSET0 = DECODER_LT; //Lamp test active low
 	  }
  }

  return 0;
}
