/******************************************************************************
 *
 * Copyright:
 *    (C) 2000 - 2009 Embedded Artists AB
 *
 * File:
 *    main.c
 *
 * Description:
 *    Sample application that demonstrates how to control a 7-segment via
 *    a decoder chip and a counter.
 *
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "general.h"
#include <lpc2xxx.h>
#include <printf_P.h>
#include <ea_init.h>
#include "startup/config.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/
#define COUNTER_CLK   0x00000100
#define COUNTER_CCKEN 0x00000200
#define COUNTER_CCLR  0x00000400
#define COUNTER_RCK   0x00000800

#define DECODER_LT    0x00001000
#define DECODER_BI    0x00002000
#define DECODER_LE    0x00004000

#define SEVEN_SEG_DOT 0x00008000

#define KEY_P1_20     0x00100000


/*****************************************************************************
 * Public function prototypes
 ****************************************************************************/
int main(void);


/*****************************************************************************
 * Local function prototypes
 ****************************************************************************/
static void delayMs(tU16 delayInMs);
static void set7seg(tU8 value);


/*****************************************************************************
 * Implementation of local functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    Delay execution by a specified number of milliseconds by using
 *    timer #1. A polled implementation.
 *
 * Params:
 *    [in] delayInMs - the number of milliseconds to delay.
 *
 ****************************************************************************/
static void
delayMs(tU16 delayInMs)
{
  /*
   * setup timer #1 for delay
   */
  TIMER1_TCR = 0x02;          //stop and reset timer
  TIMER1_PR  = 0x00;          //set prescaler to zero
  TIMER1_MR0 = delayInMs * ((FOSC * PLL_MUL)/ (1000 * PBSD));
  TIMER1_IR  = 0xff;          //reset all interrrupt flags
  TIMER1_MCR = 0x04;          //stop timer on match
  TIMER1_TCR = 0x01;          //start timer
  
  //wait until delay time has elapsed
  while (TIMER1_TCR & 0x01)
    ;
}


/*****************************************************************************
 *
 * Description:
 *
 ****************************************************************************/
static void
set7seg(tU8 value)
{
  tU8 i;

  //Reset counter
  IOSET0  =  COUNTER_CCLR;
  IOCLR0  =  COUNTER_CCLR;
  IOSET0  =  COUNTER_CCLR;

  //output value
  IOCLR0  =  COUNTER_CLK;
  for (i=0; i<value; i++)
  {
    IOSET0  =  COUNTER_CLK;
    IOCLR0  =  COUNTER_CLK;
  }
  IOSET0  =  COUNTER_RCK;
  IOCLR0  =  COUNTER_RCK;
    
  //set latch low
  IOCLR0 = DECODER_LE;

  //set latch high to store value
  IOSET0 = DECODER_LE;

}


/*****************************************************************************
 * Implementation of public functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    The main-function. 
 *
 * Returns:
 *    Always 0, since return value is not used.
 *
 ****************************************************************************/
int
main(void)
{
	tU16 counter = 0;

  //initialize printf()-functionality
  eaInit();

  //print welcome message
  printf("\n*********************************************************");
  printf("\n*");
  printf("\n* (C) 2000 - 2009 Embedded Artists AB");
  printf("\n*");
  printf("\n* Excersise #3 - drive of 7-segment display via decoder");
  printf("\n* and a counter.");
  printf("\n*");
  printf("\n*********************************************************");

  //Set controlling pins to outputs and set them to default values
  IODIR0 |= (COUNTER_CLK | COUNTER_CCKEN | COUNTER_CCLR | COUNTER_RCK |
             DECODER_LT | DECODER_BI | DECODER_LE | SEVEN_SEG_DOT);

  IOCLR0  =  COUNTER_CLK;
  IOCLR0  =  COUNTER_CCKEN; //Count enable is active low
  IOCLR0  =  COUNTER_RCK;

  //Reset counter
  IOSET0  =  COUNTER_CCLR;
  IOCLR0  =  COUNTER_CCLR;
  IOSET0  =  COUNTER_CCLR;
  
  IOSET0  =  DECODER_LT;  //Lamp test active low
  IOSET0  =  DECODER_BI;
  IOSET0  =  DECODER_LE;
  IOCLR0  =  SEVEN_SEG_DOT;

  //Set P1.20 key as input
  IODIR1 &= ~KEY_P1_20;

  //Set start value on display
  set7seg(0);

  while(1)
  {
    //wait 10 ms
    delayMs(10);    

    //update counter - count to 50 to get 500ms delay
    counter++;
    if ((counter % 50) == 0)
    {
      //check if counter has reached max (count 50 for every increment on 7-segment display)
      if (counter >= 50*10)
        counter = 0;
        
      set7seg(counter / 50);
    }
    
    //inhibit counting when P1.20 key is pressed
	  if ((IOPIN1 & KEY_P1_20) == 0)
 	  {
      IOSET0 = COUNTER_CCKEN; //Disable counter
 	  }
 	  else
 	  {
      IOCLR0 = COUNTER_CCKEN; //Enable counter
 	  }

  }
  
  return 0;
}
