/******************************************************************************
 *
 * Copyright:
 *    (C) 2000 - 2009 Embedded Artists AB
 *
 * File:
 *    main.c
 *
 * Description:
 *    Sample application that demonstrates how to control a 7-segment via
 *    a shift register by using spi-bus.
 *
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "general.h"
#include <lpc2xxx.h>
#include <printf_P.h>
#include <ea_init.h>
#include "startup/config.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/
#define SEVEN_SEG_A   0x01
#define SEVEN_SEG_B   0x02
#define SEVEN_SEG_C   0x04
#define SEVEN_SEG_D   0x08
#define SEVEN_SEG_E   0x10
#define SEVEN_SEG_F   0x20
#define SEVEN_SEG_G   0x40
#define SEVEN_SEG_DOT 0x80

//assuming bit0 -> SegA, bit1 -> segB, etc.
static const tU8 sevenSegPattern[10] = {
  (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D | SEVEN_SEG_E | SEVEN_SEG_F              ),
  (              SEVEN_SEG_B | SEVEN_SEG_C                                                        ),
  (SEVEN_SEG_A | SEVEN_SEG_B               | SEVEN_SEG_D | SEVEN_SEG_E               | SEVEN_SEG_G),
  (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D                             | SEVEN_SEG_G),
  (              SEVEN_SEG_B | SEVEN_SEG_C                             | SEVEN_SEG_F | SEVEN_SEG_G),
  (SEVEN_SEG_A               | SEVEN_SEG_C | SEVEN_SEG_D               | SEVEN_SEG_F | SEVEN_SEG_G),
  (SEVEN_SEG_A               | SEVEN_SEG_C | SEVEN_SEG_D | SEVEN_SEG_E | SEVEN_SEG_F | SEVEN_SEG_G),
  (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C                                                        ),
  (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D | SEVEN_SEG_E | SEVEN_SEG_F | SEVEN_SEG_G),
  (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D               | SEVEN_SEG_F | SEVEN_SEG_G)};

//#define SHIFTREG_SCK    0x00000010  //P0.4
//#define SHIFTREG_SERIN  0x00000020  //P0.5
//#define SHIFTREG_SEROUT 0x00000040  //P0.6
#define SHIFTREG_RCK    0x00000080  //P0.7
#define SHIFTREG_G      0x00000100  //P0.8
#define SHIFTREG_SCL    0x00000200  //P0.9

#define SPI_SLAVE_CS    SHIFTREG_RCK

/*****************************************************************************
 * Public function prototypes
 ****************************************************************************/
int main(void);


/*****************************************************************************
 * Local function prototypes
 ****************************************************************************/
static void delayMs(tU16 delayInMs);
static void set7seg(tU8 value);


/*****************************************************************************
 * Implementation of local functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    Delay execution by a specified number of milliseconds by using
 *    timer #1. A polled implementation.
 *
 * Params:
 *    [in] delayInMs - the number of milliseconds to delay.
 *
 ****************************************************************************/
static void
delayMs(tU16 delayInMs)
{
  /*
   * setup timer #1 for delay
   */
  TIMER1_TCR = 0x02;          //stop and reset timer
  TIMER1_PR  = 0x00;          //set prescaler to zero
  TIMER1_MR0 = delayInMs * ((FOSC * PLL_MUL)/ (1000 * PBSD));
  TIMER1_IR  = 0xff;          //reset all interrrupt flags
  TIMER1_MCR = 0x04;          //stop timer on match
  TIMER1_TCR = 0x01;          //start timer
  
  //wait until delay time has elapsed
  while (TIMER1_TCR & 0x01)
    ;
}


/*****************************************************************************
 *
 * Description:
 *    Sends and received one byte over the SPI serial channel.
 *    A polled implementation.
 *
 * Params:
 *    [in] byte - the byte to send.
 *
 * Return:
 *    The received byte.
 *
 ****************************************************************************/
static tU8
sendSpi(tU8 byte)
{
  SPI_SPDR = byte;
  while((SPI_SPSR & 0x80) == 0)
    ;
}

/*****************************************************************************
 *
 * Description:
 *
 ****************************************************************************/
static void
set7seg(tU8 value)
{
  tU8 pattern;

  pattern = sevenSegPattern[value & 0x0f];

  //act as ship select (not really neede)
  IOCLR0 = SPI_SLAVE_CS;

  //send byte to shift register via spi
  sendSpi(pattern);

  //transfer shift register values to output register (on positive edge)
  IOSET0 = SPI_SLAVE_CS;
}


/*****************************************************************************
 * Implementation of public functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    The main-function. 
 *
 * Returns:
 *    Always 0, since return value is not used.
 *
 ****************************************************************************/
int
main(void)
{
	tU16 counter = 0;

  //initialize printf()-functionality
  eaInit();

  //print welcome message
  printf("\n*********************************************************");
  printf("\n*");
  printf("\n* (C) 2000 - 2009 Embedded Artists AB");
  printf("\n*");
  printf("\n* Excersise #4a - drive of 7-segment display via shift-");
  printf("\n* register by using spi-bus.");
  printf("\n*");
  printf("\n*********************************************************");

  //Set controlling pins to outputs and set them to default values
  IODIR0 |= (SHIFTREG_RCK | SHIFTREG_G | SHIFTREG_SCL);
  IOCLR0  = (SHIFTREG_RCK | SHIFTREG_G);
  
  //connect SPI bus to IO-pins (note that P0.7=SPI_SLAVE_CS is still under GPIO control)
  PINSEL0 |= 0x00001500;
  
  //initialize SPI interface
  SPI_SPCCR = 0x20;    
  SPI_SPCR  = 0x20;  //MSB first, master mode, CPOL=0, CPHA=0

  //make SPI slave chip select an output and set signal high
  IODIR0 |= SPI_SLAVE_CS;
  IOSET0  = SPI_SLAVE_CS;
  
  //Pulse MR (MasterReset) to empty shift register
  IOSET0  =  SHIFTREG_SCL;
  IOCLR0  =  SHIFTREG_SCL;
  IOSET0  =  SHIFTREG_SCL;
  
  //Set start value on display
  set7seg(0);

  while(1)
  {
    //wait 10 ms
    delayMs(10);    

    //update counter - count to 50 to get 500ms delay
    counter++;
    if ((counter % 50) == 0)
    {
      //check if counter has reached max (count 50 for every increment on 7-segment display)
      if (counter >= 50*10)
        counter = 0;
        
      set7seg(counter / 50);
    }
  }
  
  return 0;
}
