/******************************************************************************
 *
 * Copyright:
 *    (C) 2000 - 2009 Embedded Artists AB
 *
 * File:
 *    main.c
 *
 * Description:
 *    Sample application that demonstrates how to communicate with ds1621 over i2c.
 *
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "general.h"
#include <lpc2xxx.h>
#include <printf_P.h>
#include <ea_init.h>
#include <framework.h>
#include "startup/config.h"
#include "i2c.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/
#define LOCAL_DS1621_ADDR 0x0         //three local address pins
#define DS1621_ADDR       0x90
#define I2C_DS1621_ADDR   (DS1621_ADDR + (LOCAL_DS1621_ADDR << 1))


/*****************************************************************************
 * Public function prototypes
 ****************************************************************************/
int main(void);


/*****************************************************************************
 * Local function prototypes
 ****************************************************************************/
static void delayMs(tU16 delayInMs);


/*****************************************************************************
 * Implementation of local functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    Delay execution by a specified number of milliseconds by using
 *    timer #1. A polled implementation.
 *
 * Params:
 *    [in] delayInMs - the number of milliseconds to delay.
 *
 ****************************************************************************/
static void
delayMs(tU16 delayInMs)
{
  /*
   * setup timer #1 for delay
   */
  TIMER1_TCR = 0x02;          //stop and reset timer
  TIMER1_PR  = 0x00;          //set prescaler to zero
  TIMER1_MR0 = delayInMs * ((FOSC * PLL_MUL)/ (1000 * PBSD));
  TIMER1_IR  = 0xff;          //reset all interrrupt flags
  TIMER1_MCR = 0x04;          //stop timer on match
  TIMER1_TCR = 0x01;          //start timer
  
  //wait until delay time has elapsed
  while (TIMER1_TCR & 0x01)
    ;
}


/*****************************************************************************
 * Implementation of public functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    The main-function. 
 *
 * Returns:
 *    Always 0, since return value is not used.
 *
 ****************************************************************************/
int
main(void)
{
	tU8 counter = '0';

  //initialize printf()-functionality
  eaInit();

  //print welcome message
  printf("\n*********************************************************");
  printf("\n*");
  printf("\n* (C) 2000 - 2009 Embedded Artists AB");
  printf("\n*");
  printf("\n* Excersise #5 - demonstrate i2c communication with DS1621");
  printf("\n*");
  printf("\n*********************************************************");

  //initialice i2c communication
  i2cInit(100000);

  //enter forever loop...
  while(1)
  {
  	tU16 length;
  	tS8  buffer[2];
  	tS8  retCode;

  	//Start temperature conversion
    retCode = i2cWrite(I2C_DS1621_ADDR, I2C_EXTRA_BYTE, 0xEE, NULL, 0);
    i2cStop();
    if (retCode == I2C_CODE_OK)
    {
      //wait 750 ms
      delayMs(750);
    
    	//Read termperature result
      retCode = i2cWrite(I2C_DS1621_ADDR, I2C_EXTRA_BYTE, 0xAA, NULL, 0);
      retCode = i2cRepeatStart();
      if (retCode == I2C_CODE_OK)
        retCode = i2cRead(I2C_DS1621_ADDR + 1, buffer, 2);
      if (retCode == I2C_CODE_OK)
      {
        tU8 subDegree;
        
        if (buffer[1] == 0x00)
          subDegree = 0;
        else
          subDegree = 5;

        if (buffer[0] <= 127)
          printf("\nRead temperature successful [%d.%d degrees Celcius]", buffer[0], subDegree);
        else
          printf("\nRead temperature successful [-%d.%d degrees Celcius]", 256-buffer[0], subDegree);

      	//Stop temperature conversion
        retCode = i2cWrite(I2C_DS1621_ADDR, I2C_EXTRA_BYTE, 0x22, NULL, 0);
        i2cStop();
      }
      else
        printf("\nRead operation failed!!!");
    }
    else
      printf("\nRead operation failed!!!");
    
  	//wait 5000 ms
    delayMs(5000);
  }

  return 0;
}
