/******************************************************************************
 *
 * Copyright:
 *    (C) 2000 - 2009 Embedded Artists AB
 *
 * File:
 *    main.c
 *
 * Description:
 *    Sample application that demonstrates how to communicate with DS1621
 *    temperature sensor over i2c and present it on a 7-segment display.
 *
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "general.h"
#include <lpc2xxx.h>
#include <printf_P.h>
#include <ea_init.h>
#include <framework.h>
#include "startup/config.h"
#include "i2c.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/
#define LOCAL_DS1621_ADDR 0x0         //three local address pins
#define DS1621_ADDR       0x90
#define I2C_DS1621_ADDR   (DS1621_ADDR + (LOCAL_DS1621_ADDR << 1))

#define SEVEN_SEG_A   0x01
#define SEVEN_SEG_B   0x02
#define SEVEN_SEG_C   0x04
#define SEVEN_SEG_D   0x08
#define SEVEN_SEG_E   0x10
#define SEVEN_SEG_F   0x20
#define SEVEN_SEG_G   0x40
#define SEVEN_SEG_DOT 0x80

//assuming bit0 -> SegA, bit1 -> segB, etc.
static const tU8 sevenSegPattern[10] = {
  (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D | SEVEN_SEG_E | SEVEN_SEG_F              ),
  (              SEVEN_SEG_B | SEVEN_SEG_C                                                        ),
  (SEVEN_SEG_A | SEVEN_SEG_B               | SEVEN_SEG_D | SEVEN_SEG_E               | SEVEN_SEG_G),
  (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D                             | SEVEN_SEG_G),
  (              SEVEN_SEG_B | SEVEN_SEG_C                             | SEVEN_SEG_F | SEVEN_SEG_G),
  (SEVEN_SEG_A               | SEVEN_SEG_C | SEVEN_SEG_D               | SEVEN_SEG_F | SEVEN_SEG_G),
  (SEVEN_SEG_A               | SEVEN_SEG_C | SEVEN_SEG_D | SEVEN_SEG_E | SEVEN_SEG_F | SEVEN_SEG_G),
  (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C                                                        ),
  (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D | SEVEN_SEG_E | SEVEN_SEG_F | SEVEN_SEG_G),
  (SEVEN_SEG_A | SEVEN_SEG_B | SEVEN_SEG_C | SEVEN_SEG_D               | SEVEN_SEG_F | SEVEN_SEG_G)};

//#define SHIFTREG_SCK    0x00000010  //P0.4
//#define SHIFTREG_SERIN  0x00000020  //P0.5
//#define SHIFTREG_SEROUT 0x00000040  //P0.6
#define SHIFTREG_RCK    0x00000080  //P0.7
#define SHIFTREG_G      0x00000100  //P0.8
#define SHIFTREG_SCL    0x00000200  //P0.9

#define SPI_SLAVE_CS    SHIFTREG_RCK

#define CMD_DIGIT 0
#define CMD_BLANK 1
#define CMD_MINUS 2
#define CMD_DOT   3

#define DISP_DELAY 500


/*****************************************************************************
 * Public function prototypes
 ****************************************************************************/
int main(void);


/*****************************************************************************
 * Local function prototypes
 ****************************************************************************/
static void delayMs(tU16 delayInMs);
static tU8 sendSpi(tU8 byte);
static void set7seg(tU8 value, tU8 blankFlag);

/*****************************************************************************
 * Implementation of local functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    Delay execution by a specified number of milliseconds by using
 *    timer #1. A polled implementation.
 *
 * Params:
 *    [in] delayInMs - the number of milliseconds to delay.
 *
 ****************************************************************************/
static void
delayMs(tU16 delayInMs)
{
  /*
   * setup timer #1 for delay
   */
  TIMER1_TCR = 0x02;          //stop and reset timer
  TIMER1_PR  = 0x00;          //set prescaler to zero
  TIMER1_MR0 = delayInMs * ((FOSC * PLL_MUL)/ (1000 * PBSD));
  TIMER1_IR  = 0xff;          //reset all interrrupt flags
  TIMER1_MCR = 0x04;          //stop timer on match
  TIMER1_TCR = 0x01;          //start timer
  
  //wait until delay time has elapsed
  while (TIMER1_TCR & 0x01)
    ;
}

/*****************************************************************************
 *
 * Description:
 *    Sends and received one byte over the SPI serial channel.
 *    A polled implementation.
 *
 * Params:
 *    [in] byte - the byte to send.
 *
 * Return:
 *    The received byte.
 *
 ****************************************************************************/
static tU8
sendSpi(tU8 byte)
{
  SPI_SPDR = byte;
  while((SPI_SPSR & 0x80) == 0)
    ;
}

/*****************************************************************************
 *
 * Description:
 *
 ****************************************************************************/
static void
set7seg(tU8 value, tU8 command)
{
  tU8 pattern;

  if (command == CMD_BLANK)
    pattern = 0;
  else if (command == CMD_MINUS)
    pattern = 0x40;
  else if (command == CMD_DOT)
    pattern = 0x80;
  else
    pattern = sevenSegPattern[value & 0x0f];

  //act as ship select (not really neede)
  IOCLR0 = SPI_SLAVE_CS;

  //send byte to shift register via spi
  sendSpi(pattern);

  //transfer shift register values to output register (on positive edge)
  IOSET0 = SPI_SLAVE_CS;
}


/*****************************************************************************
 * Implementation of public functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    The main-function. 
 *
 * Returns:
 *    Always 0, since return value is not used.
 *
 ****************************************************************************/
int
main(void)
{
	tU8 counter = '0';

  //initialize printf()-functionality
  eaInit();

  //print welcome message
  printf("\n*********************************************************");
  printf("\n*");
  printf("\n* (C) 2000 - 2009 Embedded Artists AB");
  printf("\n*");
  printf("\n* Excersise #5a - demonstrate i2c communication with DS1621");
  printf("\n* temperature sensor and present result on 7-segment display");
  printf("\n*");
  printf("\n*********************************************************");

  //initialice i2c communication
  i2cInit(100000);

  //Set controlling pins to outputs and set them to default values
  IODIR0 |= (SHIFTREG_RCK | SHIFTREG_G | SHIFTREG_SCL);
  IOCLR0  = (SHIFTREG_RCK | SHIFTREG_G);
  
  //connect SPI bus to IO-pins (note that P0.7=SPI_SLAVE_CS is still under GPIO control)
  PINSEL0 |= 0x00001500;
  
  //initialize SPI interface
  SPI_SPCCR = 0x20;    
  SPI_SPCR  = 0x20;  //MSB first, master mode, CPOL=0, CPHA=0

  //make SPI slave chip select an output and set signal high
  IODIR0 |= SPI_SLAVE_CS;
  IOSET0  = SPI_SLAVE_CS;
  
  //Pulse MR (MasterReset) to empty shift register
  IOSET0  =  SHIFTREG_SCL;
  IOCLR0  =  SHIFTREG_SCL;
  IOSET0  =  SHIFTREG_SCL;
  
  set7seg(0, CMD_BLANK);

  //enter forever loop...
  while(1)
  {
  	tU16 length;
  	tS8  buffer[2];
  	tS8  retCode;

  	//Start temperature conversion
    retCode = i2cWrite(I2C_DS1621_ADDR, I2C_EXTRA_BYTE, 0xEE, NULL, 0);
    i2cStop();
    if (retCode == I2C_CODE_OK)
    {
      //wait 750 ms
      delayMs(750);
    
    	//Read termperature result
      retCode = i2cWrite(I2C_DS1621_ADDR, I2C_EXTRA_BYTE, 0xAA, NULL, 0);
      retCode = i2cRepeatStart();
      if (retCode == I2C_CODE_OK)
        retCode = i2cRead(I2C_DS1621_ADDR + 1, buffer, 2);
      if (retCode == I2C_CODE_OK)
      {
        tU8 subDegree;
        
        if (buffer[1] == 0x00)
          subDegree = 0;
        else
          subDegree = 5;

        if (buffer[0] <= 127)
        {
          printf("\nRead temperature successful [%d.%d degrees Celcius]", buffer[0], subDegree);
          set7seg(buffer[0] / 10, CMD_DIGIT);
          delayMs(DISP_DELAY);
          set7seg(buffer[0] % 10, CMD_DIGIT);
          delayMs(DISP_DELAY);
          set7seg(0, CMD_DOT);
          delayMs(DISP_DELAY);
          set7seg(subDegree, CMD_DIGIT);
          delayMs(DISP_DELAY);
          set7seg(0, CMD_BLANK);
        }
        else
        {
          printf("\nRead temperature successful [-%d.%d degrees Celcius]", 256-buffer[0], subDegree);
          set7seg(0, CMD_MINUS);
          delayMs(DISP_DELAY);
          set7seg((256 - buffer[0]) / 10, CMD_DIGIT);
          delayMs(DISP_DELAY);
          set7seg((256 - buffer[0]) % 10, CMD_DIGIT);
          delayMs(DISP_DELAY);
          set7seg(0, CMD_DOT);
          delayMs(DISP_DELAY);
          set7seg(subDegree, CMD_DIGIT);
          delayMs(DISP_DELAY);
          set7seg(0, CMD_BLANK);
        }

      	//Stop temperature conversion
        retCode = i2cWrite(I2C_DS1621_ADDR, I2C_EXTRA_BYTE, 0x22, NULL, 0);
        i2cStop();
      }
      else
        printf("\nRead operation failed!!!");
    }
    else
      printf("\nRead operation failed!!!");
    
  	//wait 5000 ms
    delayMs(5000);
  }

  return 0;
}
