/******************************************************************************
 *
 * Copyright:
 *    (C) 2000 - 2009 Embedded Artists AB
 *
 * File:
 *    main.c
 *
 * Description:
 *    Sample application that demonstrates how to communicate with CAT1025 over i2c.
 *
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "general.h"
#include <lpc2xxx.h>
#include <printf_P.h>
#include <ea_init.h>
#include <framework.h>
#include "startup/config.h"
#include "i2c.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/
#define LOCAL_EEPROM_ADDR 0x0         //if EEPROM has local address pins
#define EEPROM_ADDR       0xA0
#define I2C_EEPROM_ADDR   (EEPROM_ADDR + (LOCAL_EEPROM_ADDR << 1))


/*****************************************************************************
 * Public function prototypes
 ****************************************************************************/
int main(void);


/*****************************************************************************
 * Local function prototypes
 ****************************************************************************/
static void delayMs(tU16 delayInMs);


/*****************************************************************************
 * Implementation of local functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    Delay execution by a specified number of milliseconds by using
 *    timer #1. A polled implementation.
 *
 * Params:
 *    [in] delayInMs - the number of milliseconds to delay.
 *
 ****************************************************************************/
static void
delayMs(tU16 delayInMs)
{
  /*
   * setup timer #1 for delay
   */
  TIMER1_TCR = 0x02;          //stop and reset timer
  TIMER1_PR  = 0x00;          //set prescaler to zero
  TIMER1_MR0 = delayInMs * ((FOSC * PLL_MUL)/ (1000 * PBSD));
  TIMER1_IR  = 0xff;          //reset all interrrupt flags
  TIMER1_MCR = 0x04;          //stop timer on match
  TIMER1_TCR = 0x01;          //start timer
  
  //wait until delay time has elapsed
  while (TIMER1_TCR & 0x01)
    ;
}


/*****************************************************************************
 * Implementation of public functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    The main-function. 
 *
 * Returns:
 *    Always 0, since return value is not used.
 *
 ****************************************************************************/
int
main(void)
{
	tU8 counter = 0;

  //initialize printf()-functionality
  eaInit();

  //print welcome message
  printf("\n*********************************************************");
  printf("\n*");
  printf("\n* (C) 2000 - 2009 Embedded Artists AB");
  printf("\n*");
  printf("\n* Excersise #5b - demonstrate i2c communication with");
  printf("\n* CAT1025 - E2PROM");
  printf("\n*");
  printf("\n*********************************************************");

  //initialice i2c communication
  i2cInit(100000);

  //enter forever loop...
  while(1)
  {
  	tU16 length;
  	tU8  buffer[20];
  	volatile tS8 retCode;

  	/*
  	 * read from EEPROM at address 0x00 with one address byte
  	 */
  	do
  	{
    	//First write address to e2prom
    	length = 0;
      retCode = i2cWrite(I2C_EEPROM_ADDR, I2C_EXTRA_BYTE, 0x00, buffer, length);
      if(retCode != I2C_CODE_OK)
        break;
        
      retCode = i2cRepeatStart();
      if(retCode != I2C_CODE_OK)
        break;
      
      //After that, start read operation
      retCode = i2cRead(I2C_EEPROM_ADDR + 1, buffer, sizeof(buffer)-1);
      if(retCode != I2C_CODE_OK)
        break;
      buffer[19] = 0x00; //just in case the read string is not properly null-terminated
    } while (0);

    if (retCode == I2C_CODE_OK)
      printf("\nRead operation  [%s] status: I2C_CODE_OK", buffer);
    else
      printf("\nRead operation failed!!!");
  	
  	/*
  	 * write to EEPROM at address 0x00 with one address byte
  	 */
  	buffer[0]  = 't';
  	buffer[1]  = 'e';
  	buffer[2]  = 's';
  	buffer[3]  = 't';
  	buffer[4]  = ' ';
  	buffer[5]  = 's';
  	buffer[6]  = 't';
  	buffer[7]  = 'r';
  	buffer[8]  = 'i';
  	buffer[9]  = 'n';
  	buffer[10] = 'g';
  	buffer[11] = ':';
  	buffer[12] = ' ';
  	buffer[13] = '0' + (counter / 10);
  	buffer[14] = '0' + (counter % 10);
  	buffer[15] = 0x00;
  	length = 16;
  	counter++;
  	
    retCode = i2cWrite(I2C_EEPROM_ADDR, I2C_EXTRA_BYTE, 0x00, buffer, length);
    i2cStop();
    if (retCode == I2C_CODE_OK)
      printf("\nWrite operation [%s] status: I2C_CODE_OK", buffer, retCode);
    else
      printf("\nWrite operation failed!!!");

    retCode = i2cPoll(I2C_EEPROM_ADDR);
    if (retCode != I2C_CODE_OK)
      printf(" ... and E2PROM is not ready = ERROR!!!");

  	//wait 10000 ms
    delayMs(10000);
  }

  return 0;
}
